@@ -75,6 +75,7 @@ impl ZellijPlugin for State {
         }
         let mut all_tabs: Vec<LinePart> = vec![];
         let mut active_tab_index = 0;
+        let mut is_alternate_tab = false;
         for t in &mut self.tabs {
             let mut tabname = t.name.clone();
             if t.active && self.mode_info.mode == InputMode::RenameTab {
@@ -88,11 +89,13 @@ impl ZellijPlugin for State {
             let tab = tab_style(
                 tabname,
                 t.active,
+                is_alternate_tab,
                 t.is_sync_panes_active,
                 self.mode_info.style.colors,
                 self.mode_info.capabilities,
                 t.other_focused_clients.as_slice(),
             );
+            is_alternate_tab = !is_alternate_tab;
             all_tabs.push(tab);
         }
         let tab_line = tab_line(
