@@ -18,7 +18,7 @@ use std::time::Instant;
 use zellij_tile::data::ModeInfo;
 use zellij_utils::{
     input::layout::Direction,
-    pane_size::{Offset, PaneGeom, Size, Viewport},
+    pane_size::{Offset, PaneGeom, Size, SizeInPixels, Viewport},
 };
 
 macro_rules! resize_pty {
@@ -61,6 +61,7 @@ pub struct TiledPanes {
     connected_clients: Rc<RefCell<HashSet<ClientId>>>,
     connected_clients_in_app: Rc<RefCell<HashSet<ClientId>>>,
     mode_info: Rc<RefCell<HashMap<ClientId, ModeInfo>>>,
+    character_cell_size: Rc<RefCell<Option<SizeInPixels>>>,
     default_mode_info: ModeInfo,
     style: Style,
     session_is_mirrored: bool,
@@ -79,6 +80,7 @@ impl TiledPanes {
         connected_clients: Rc<RefCell<HashSet<ClientId>>>,
         connected_clients_in_app: Rc<RefCell<HashSet<ClientId>>>,
         mode_info: Rc<RefCell<HashMap<ClientId, ModeInfo>>>,
+        character_cell_size: Rc<RefCell<Option<SizeInPixels>>>,
         session_is_mirrored: bool,
         draw_pane_frames: bool,
         default_mode_info: ModeInfo,
@@ -92,6 +94,7 @@ impl TiledPanes {
             connected_clients,
             connected_clients_in_app,
             mode_info,
+            character_cell_size,
             default_mode_info,
             style,
             session_is_mirrored,
@@ -102,16 +105,63 @@ impl TiledPanes {
             os_api,
         }
     }
-    pub fn add_pane_with_existing_geom(&mut self, pane_id: PaneId, pane: Box<dyn Pane>) {
+    pub fn add_pane_with_existing_geom(&mut self, pane_id: PaneId, mut pane: Box<dyn Pane>) {
+        if self.draw_pane_frames {
+            pane.set_content_offset(Offset::frame(1));
+        }
         self.panes.insert(pane_id, pane);
     }
+    pub fn replace_active_pane(
+        &mut self,
+        pane: Box<dyn Pane>,
+        client_id: ClientId,
+    ) -> Option<Box<dyn Pane>> {
+        let pane_id = pane.pid();
+        // remove the currently active pane
+        let previously_active_pane = self
+            .active_panes
+            .get(&client_id)
+            .copied()
+            .and_then(|active_pane_id| self.replace_pane(active_pane_id, pane));
+
+        // move clients from the previously active pane to the new pane we just inserted
+        if let Some(previously_active_pane) = previously_active_pane.as_ref() {
+            let previously_active_pane_id = previously_active_pane.pid();
+            self.move_clients_between_panes(previously_active_pane_id, pane_id);
+        }
+        previously_active_pane
+    }
+    pub fn replace_pane(
+        &mut self,
+        pane_id: PaneId,
+        mut with_pane: Box<dyn Pane>,
+    ) -> Option<Box<dyn Pane>> {
+        let with_pane_id = with_pane.pid();
+        if self.draw_pane_frames {
+            with_pane.set_content_offset(Offset::frame(1));
+        }
+        let removed_pane = self.panes.remove(&pane_id).map(|removed_pane| {
+            let with_pane_id = with_pane.pid();
+            let removed_pane_geom = removed_pane.current_geom();
+            with_pane.set_geom(removed_pane_geom);
+            self.panes.insert(with_pane_id, with_pane);
+            removed_pane
+        });
+
+        // move clients from the previously active pane to the new pane we just inserted
+        self.move_clients_between_panes(pane_id, with_pane_id);
+        removed_pane
+    }
     pub fn insert_pane(&mut self, pane_id: PaneId, mut pane: Box<dyn Pane>) {
+        let cursor_height_width_ratio = self.cursor_height_width_ratio();
         let pane_grid = TiledPaneGrid::new(
             &mut self.panes,
+            &self.panes_to_hide,
             *self.display_area.borrow(),
             *self.viewport.borrow(),
         );
-        let pane_id_and_split_direction = pane_grid.find_room_for_new_pane();
+        let pane_id_and_split_direction =
+            pane_grid.find_room_for_new_pane(cursor_height_width_ratio);
         if let Some((pane_id_to_split, split_direction)) = pane_id_and_split_direction {
             // this unwrap is safe because floating panes should not be visible if there are no floating panes
             let pane_to_split = self.panes.get_mut(&pane_id_to_split).unwrap();
@@ -125,12 +175,16 @@ impl TiledPanes {
         }
     }
     pub fn has_room_for_new_pane(&mut self) -> bool {
+        let cursor_height_width_ratio = self.cursor_height_width_ratio();
         let pane_grid = TiledPaneGrid::new(
             &mut self.panes,
+            &self.panes_to_hide,
             *self.display_area.borrow(),
             *self.viewport.borrow(),
         );
-        pane_grid.find_room_for_new_pane().is_some()
+        pane_grid
+            .find_room_for_new_pane(cursor_height_width_ratio)
+            .is_some()
     }
     pub fn fixed_pane_geoms(&self) -> Vec<Viewport> {
         self.panes
@@ -158,6 +212,7 @@ impl TiledPanes {
     pub fn relayout(&mut self, direction: Direction) {
         let mut pane_grid = TiledPaneGrid::new(
             &mut self.panes,
+            &self.panes_to_hide,
             *self.display_area.borrow(),
             *self.viewport.borrow(),
         );
@@ -391,12 +446,13 @@ impl TiledPanes {
         {
             let mut display_area = self.display_area.borrow_mut();
             let mut viewport = self.viewport.borrow_mut();
-            let panes = self
-                .panes
-                .iter_mut()
-                .filter(|(pid, _)| !self.panes_to_hide.contains(pid));
             let Size { rows, cols } = new_screen_size;
-            let mut pane_grid = TiledPaneGrid::new(panes, *display_area, *viewport);
+            let mut pane_grid = TiledPaneGrid::new(
+                &mut self.panes,
+                &self.panes_to_hide,
+                *display_area,
+                *viewport,
+            );
             if pane_grid.layout(Direction::Horizontal, cols).is_ok() {
                 let column_difference = cols as isize - display_area.cols as isize;
                 // FIXME: Should the viewport be an Offset?
@@ -419,6 +475,7 @@ impl TiledPanes {
         if let Some(active_pane_id) = self.get_active_pane_id(client_id) {
             let mut pane_grid = TiledPaneGrid::new(
                 &mut self.panes,
+                &self.panes_to_hide,
                 *self.display_area.borrow(),
                 *self.viewport.borrow(),
             );
@@ -432,6 +489,7 @@ impl TiledPanes {
         if let Some(active_pane_id) = self.get_active_pane_id(client_id) {
             let mut pane_grid = TiledPaneGrid::new(
                 &mut self.panes,
+                &self.panes_to_hide,
                 *self.display_area.borrow(),
                 *self.viewport.borrow(),
             );
@@ -445,6 +503,7 @@ impl TiledPanes {
         if let Some(active_pane_id) = self.get_active_pane_id(client_id) {
             let mut pane_grid = TiledPaneGrid::new(
                 &mut self.panes,
+                &self.panes_to_hide,
                 *self.display_area.borrow(),
                 *self.viewport.borrow(),
             );
@@ -458,6 +517,7 @@ impl TiledPanes {
         if let Some(active_pane_id) = self.get_active_pane_id(client_id) {
             let mut pane_grid = TiledPaneGrid::new(
                 &mut self.panes,
+                &self.panes_to_hide,
                 *self.display_area.borrow(),
                 *self.viewport.borrow(),
             );
@@ -471,6 +531,7 @@ impl TiledPanes {
         if let Some(active_pane_id) = self.get_active_pane_id(client_id) {
             let mut pane_grid = TiledPaneGrid::new(
                 &mut self.panes,
+                &self.panes_to_hide,
                 *self.display_area.borrow(),
                 *self.viewport.borrow(),
             );
@@ -484,6 +545,7 @@ impl TiledPanes {
         if let Some(active_pane_id) = self.get_active_pane_id(client_id) {
             let mut pane_grid = TiledPaneGrid::new(
                 &mut self.panes,
+                &self.panes_to_hide,
                 *self.display_area.borrow(),
                 *self.viewport.borrow(),
             );
@@ -499,6 +561,7 @@ impl TiledPanes {
         let active_pane_id = self.get_active_pane_id(client_id).unwrap();
         let pane_grid = TiledPaneGrid::new(
             &mut self.panes,
+            &self.panes_to_hide,
             *self.display_area.borrow(),
             *self.viewport.borrow(),
         );
@@ -514,6 +577,7 @@ impl TiledPanes {
         let active_pane_id = self.get_active_pane_id(client_id).unwrap();
         let pane_grid = TiledPaneGrid::new(
             &mut self.panes,
+            &self.panes_to_hide,
             *self.display_area.borrow(),
             *self.viewport.borrow(),
         );
@@ -528,11 +592,18 @@ impl TiledPanes {
             pane.set_active_at(Instant::now());
         }
     }
+    pub fn cursor_height_width_ratio(&self) -> Option<usize> {
+        let character_cell_size = self.character_cell_size.borrow();
+        character_cell_size.map(|size_in_pixels| {
+            (size_in_pixels.height as f64 / size_in_pixels.width as f64).round() as usize
+        })
+    }
     pub fn move_focus_left(&mut self, client_id: ClientId) -> bool {
         match self.get_active_pane_id(client_id) {
             Some(active_pane_id) => {
                 let pane_grid = TiledPaneGrid::new(
                     &mut self.panes,
+                    &self.panes_to_hide,
                     *self.display_area.borrow(),
                     *self.viewport.borrow(),
                 );
@@ -573,6 +644,7 @@ impl TiledPanes {
             Some(active_pane_id) => {
                 let pane_grid = TiledPaneGrid::new(
                     &mut self.panes,
+                    &self.panes_to_hide,
                     *self.display_area.borrow(),
                     *self.viewport.borrow(),
                 );
@@ -613,6 +685,7 @@ impl TiledPanes {
             Some(active_pane_id) => {
                 let pane_grid = TiledPaneGrid::new(
                     &mut self.panes,
+                    &self.panes_to_hide,
                     *self.display_area.borrow(),
                     *self.viewport.borrow(),
                 );
@@ -653,6 +726,7 @@ impl TiledPanes {
             Some(active_pane_id) => {
                 let pane_grid = TiledPaneGrid::new(
                     &mut self.panes,
+                    &self.panes_to_hide,
                     *self.display_area.borrow(),
                     *self.viewport.borrow(),
                 );
@@ -692,6 +766,7 @@ impl TiledPanes {
         let active_pane_id = self.get_active_pane_id(client_id).unwrap();
         let pane_grid = TiledPaneGrid::new(
             &mut self.panes,
+            &self.panes_to_hide,
             *self.display_area.borrow(),
             *self.viewport.borrow(),
         );
@@ -722,6 +797,7 @@ impl TiledPanes {
         if let Some(active_pane_id) = self.get_active_pane_id(client_id) {
             let pane_grid = TiledPaneGrid::new(
                 &mut self.panes,
+                &self.panes_to_hide,
                 *self.display_area.borrow(),
                 *self.viewport.borrow(),
             );
@@ -756,6 +832,7 @@ impl TiledPanes {
         if let Some(active_pane_id) = self.get_active_pane_id(client_id) {
             let pane_grid = TiledPaneGrid::new(
                 &mut self.panes,
+                &self.panes_to_hide,
                 *self.display_area.borrow(),
                 *self.viewport.borrow(),
             );
@@ -790,6 +867,7 @@ impl TiledPanes {
         if let Some(active_pane_id) = self.get_active_pane_id(client_id) {
             let pane_grid = TiledPaneGrid::new(
                 &mut self.panes,
+                &self.panes_to_hide,
                 *self.display_area.borrow(),
                 *self.viewport.borrow(),
             );
@@ -824,6 +902,7 @@ impl TiledPanes {
         if let Some(active_pane_id) = self.get_active_pane_id(client_id) {
             let pane_grid = TiledPaneGrid::new(
                 &mut self.panes,
+                &self.panes_to_hide,
                 *self.display_area.borrow(),
                 *self.viewport.borrow(),
             );
@@ -863,6 +942,7 @@ impl TiledPanes {
         match self
             .panes
             .iter()
+            .filter(|(p_id, _)| !self.panes_to_hide.contains(p_id))
             .find(|(p_id, p)| **p_id != pane_id && p.selectable())
             .map(|(p_id, _p)| p_id)
         {
@@ -876,9 +956,13 @@ impl TiledPanes {
             None => self.active_panes.clear(),
         }
     }
+    pub fn extract_pane(&mut self, pane_id: PaneId) -> Option<Box<dyn Pane>> {
+        self.panes.remove(&pane_id)
+    }
     pub fn remove_pane(&mut self, pane_id: PaneId) -> Option<Box<dyn Pane>> {
         let mut pane_grid = TiledPaneGrid::new(
             &mut self.panes,
+            &self.panes_to_hide,
             *self.display_area.borrow(),
             *self.viewport.borrow(),
         );
@@ -1001,6 +1085,24 @@ impl TiledPanes {
     pub fn panes_to_hide_count(&self) -> usize {
         self.panes_to_hide.len()
     }
+    pub fn add_to_hidden_panels(&mut self, pid: PaneId) {
+        self.panes_to_hide.insert(pid);
+    }
+    pub fn remove_from_hidden_panels(&mut self, pid: PaneId) {
+        self.panes_to_hide.remove(&pid);
+    }
+    fn move_clients_between_panes(&mut self, from_pane_id: PaneId, to_pane_id: PaneId) {
+        let clients_in_pane: Vec<ClientId> = self
+            .active_panes
+            .iter()
+            .filter(|(_cid, pid)| **pid == from_pane_id)
+            .map(|(cid, _pid)| *cid)
+            .collect();
+        for client_id in clients_in_pane {
+            self.active_panes.remove(&client_id);
+            self.active_panes.insert(client_id, to_pane_id);
+        }
+    }
 }
 
 #[allow(clippy::borrowed_box)]
