@@ -1,4 +1,4 @@
-use super::{Screen, ScreenInstruction};
+use super::{CopyOptions, Screen, ScreenInstruction};
 use crate::panes::PaneId;
 use crate::zellij_tile::data::{ModeInfo, Palette};
 use crate::{
@@ -10,13 +10,12 @@ use std::convert::TryInto;
 use std::path::PathBuf;
 use zellij_utils::input::command::TerminalAction;
 use zellij_utils::input::layout::LayoutTemplate;
-use zellij_utils::input::options::Clipboard;
 use zellij_utils::ipc::IpcReceiverWithContext;
-use zellij_utils::pane_size::Size;
+use zellij_utils::pane_size::{Size, SizeInPixels};
 
 use std::os::unix::io::RawFd;
 
-use zellij_utils::ipc::ClientAttributes;
+use zellij_utils::ipc::{ClientAttributes, PixelDimensions};
 use zellij_utils::nix;
 
 use zellij_utils::{
@@ -35,7 +34,8 @@ impl ServerOsApi for FakeInputOutput {
         &self,
         _file_to_open: TerminalAction,
         _quit_db: Box<dyn Fn(PaneId) + Send>,
-    ) -> (RawFd, RawFd) {
+        _default_editor: Option<PathBuf>,
+    ) -> Result<(RawFd, RawFd), &'static str> {
         unimplemented!()
     }
     fn read_from_tty_stdout(&self, _fd: RawFd, _buf: &mut [u8]) -> Result<usize, nix::Error> {
@@ -78,6 +78,9 @@ impl ServerOsApi for FakeInputOutput {
     fn get_cwd(&self, _pid: Pid) -> Option<PathBuf> {
         unimplemented!()
     }
+    fn write_to_file(&mut self, _: String, _: Option<String>) {
+        unimplemented!()
+    }
 }
 
 fn create_new_screen(size: Size) -> Screen {
@@ -92,8 +95,8 @@ fn create_new_screen(size: Size) -> Screen {
     let mode_info = ModeInfo::default();
     let draw_pane_frames = false;
     let session_is_mirrored = true;
-    let copy_command = None;
-    let copy_clipboard = Clipboard::default();
+    let copy_options = CopyOptions::default();
+
     Screen::new(
         bus,
         &client_attributes,
@@ -101,8 +104,7 @@ fn create_new_screen(size: Size) -> Screen {
         mode_info,
         draw_pane_frames,
         session_is_mirrored,
-        copy_command,
-        copy_clipboard,
+        copy_options,
     )
 }
 
@@ -464,3 +466,81 @@ fn switch_to_tab_with_fullscreen() {
         "Active pane is still the fullscreen pane"
     );
 }
+
+#[test]
+fn update_screen_pixel_dimensions() {
+    let size = Size {
+        cols: 121,
+        rows: 20,
+    };
+    let mut screen = create_new_screen(size);
+    let initial_pixel_dimensions = screen.pixel_dimensions;
+    screen.update_pixel_dimensions(PixelDimensions {
+        character_cell_size: Some(SizeInPixels {
+            height: 10,
+            width: 5,
+        }),
+        text_area_size: None,
+    });
+    let pixel_dimensions_after_first_update = screen.pixel_dimensions;
+    screen.update_pixel_dimensions(PixelDimensions {
+        character_cell_size: None,
+        text_area_size: Some(SizeInPixels {
+            height: 100,
+            width: 50,
+        }),
+    });
+    let pixel_dimensions_after_second_update = screen.pixel_dimensions;
+    screen.update_pixel_dimensions(PixelDimensions {
+        character_cell_size: None,
+        text_area_size: None,
+    });
+    let pixel_dimensions_after_third_update = screen.pixel_dimensions;
+    assert_eq!(
+        initial_pixel_dimensions,
+        PixelDimensions {
+            character_cell_size: None,
+            text_area_size: None
+        },
+        "Initial pixel dimensions empty"
+    );
+    assert_eq!(
+        pixel_dimensions_after_first_update,
+        PixelDimensions {
+            character_cell_size: Some(SizeInPixels {
+                height: 10,
+                width: 5
+            }),
+            text_area_size: None
+        },
+        "character_cell_size updated properly",
+    );
+    assert_eq!(
+        pixel_dimensions_after_second_update,
+        PixelDimensions {
+            character_cell_size: Some(SizeInPixels {
+                height: 10,
+                width: 5
+            }),
+            text_area_size: Some(SizeInPixels {
+                height: 100,
+                width: 50,
+            }),
+        },
+        "text_area_size updated properly without overriding character_cell_size",
+    );
+    assert_eq!(
+        pixel_dimensions_after_third_update,
+        PixelDimensions {
+            character_cell_size: Some(SizeInPixels {
+                height: 10,
+                width: 5
+            }),
+            text_area_size: Some(SizeInPixels {
+                height: 100,
+                width: 50,
+            }),
+        },
+        "empty update does not delete existing data",
+    );
+}
