@@ -182,6 +182,7 @@ pub enum ContextType {
     IPCServer(ServerContext),
     StdinHandler,
     AsyncTask,
+    PtyWrite(PtyWriteContext),
     /// An empty, placeholder call. This should be thought of as representing no call at all.
     /// A call stack representation filled with these is the representation of an empty call stack.
     Empty,
@@ -197,6 +198,7 @@ impl Display for ContextType {
             ContextType::IPCServer(c) => Some(("ipc_server:", format!("{:?}", c))),
             ContextType::StdinHandler => Some(("stdin_handler_thread:", "AcceptInput".to_string())),
             ContextType::AsyncTask => Some(("stream_terminal_bytes:", "AsyncTask".to_string())),
+            ContextType::PtyWrite(c) => Some(("pty_writer_thread:", format!("{:?}", c))),
             ContextType::Empty => None,
         } {
             write!(f, "{} {}", left.purple(), right.green())
@@ -213,6 +215,7 @@ pub enum ScreenContext {
     HandlePtyBytes,
     Render,
     NewPane,
+    OpenInPlaceEditor,
     ToggleFloatingPanes,
     TogglePaneEmbedOrFloating,
     HorizontalSplit,
@@ -240,6 +243,8 @@ pub enum ScreenContext {
     MovePaneRight,
     MovePaneLeft,
     Exit,
+    DumpScreen,
+    EditScrollback,
     ScrollUp,
     ScrollUpAt,
     ScrollDown,
@@ -289,6 +294,7 @@ pub enum ScreenContext {
 #[derive(Debug, Clone, Copy, PartialEq, Serialize, Deserialize)]
 pub enum PtyContext {
     SpawnTerminal,
+    OpenInPlaceEditor,
     SpawnTerminalVertically,
     SpawnTerminalHorizontally,
     UpdateActivePane,
@@ -337,3 +343,9 @@ pub enum ServerContext {
     AttachClient,
     ConnStatus,
 }
+
+#[derive(Debug, Clone, Copy, PartialEq, Serialize, Deserialize)]
+pub enum PtyWriteContext {
+    Write,
+    Exit,
+}
