@@ -54,7 +54,10 @@ pub enum Action {
     /// If there is no pane in the direction, move to previous/next Tab.
     MoveFocusOrTab(Direction),
     MovePane(Option<Direction>),
+    /// Dumps the screen to a file
+    DumpScreen(String),
     /// Scroll up in focus pane.
+    EditScrollback,
     ScrollUp,
     /// Scroll up at point
     ScrollUpAt(Position),
