@@ -37,7 +37,9 @@ use url::Url;
 #[derive(Debug, PartialEq, Eq, Serialize, Deserialize, Clone, Copy)]
 #[serde(crate = "self::serde")]
 pub enum Direction {
+    #[serde(alias = "horizontal")]
     Horizontal,
+    #[serde(alias = "vertical")]
     Vertical,
 }
 
@@ -55,7 +57,9 @@ impl Not for Direction {
 #[derive(Debug, Serialize, Deserialize, Clone, Copy, PartialEq)]
 #[serde(crate = "self::serde")]
 pub enum SplitSize {
+    #[serde(alias = "percent")]
     Percent(f64), // 1 to 100
+    #[serde(alias = "fixed")]
     Fixed(usize), // An absolute number of columns or rows
 }
 
@@ -140,6 +144,7 @@ pub struct Layout {
     pub run: Option<Run>,
     #[serde(default)]
     pub borderless: bool,
+    pub focus: Option<bool>,
 }
 
 // The struct that is used to deserialize the layout from
@@ -234,12 +239,21 @@ impl LayoutFromYamlIntermediate {
 
     pub fn from_path_or_default(
         layout: Option<&PathBuf>,
-        layout_path: Option<&PathBuf>,
         layout_dir: Option<PathBuf>,
     ) -> Option<LayoutFromYamlIntermediateResult> {
         layout
-            .map(|p| LayoutFromYamlIntermediate::from_dir(p, layout_dir.as_ref()))
-            .or_else(|| layout_path.map(|p| LayoutFromYamlIntermediate::from_path(p)))
+            .map(|layout| {
+                // The way we determine where to look for the layout is similar to
+                // how a path would look for an executable.
+                // See the gh issue for more: https://github.com/zellij-org/zellij/issues/1412#issuecomment-1131559720
+                if layout.extension().is_some() || layout.components().count() > 1 {
+                    // We look localy!
+                    LayoutFromYamlIntermediate::from_path(layout)
+                } else {
+                    // We look in the default dir
+                    LayoutFromYamlIntermediate::from_dir(layout, layout_dir.as_ref())
+                }
+            })
             .or_else(|| {
                 Some(LayoutFromYamlIntermediate::from_dir(
                     &std::path::PathBuf::from("default"),
@@ -255,8 +269,14 @@ impl LayoutFromYamlIntermediate {
         layout_dir: Option<&PathBuf>,
     ) -> LayoutFromYamlIntermediateResult {
         match layout_dir {
-            Some(dir) => Self::from_path(&dir.join(layout))
-                .or_else(|_| LayoutFromYamlIntermediate::from_default_assets(layout)),
+            Some(dir) => {
+                let layout_path = &dir.join(layout);
+                if layout_path.with_extension("yaml").exists() {
+                    Self::from_path(layout_path)
+                } else {
+                    LayoutFromYamlIntermediate::from_default_assets(layout)
+                }
+            }
             None => LayoutFromYamlIntermediate::from_default_assets(layout),
         }
     }
@@ -268,6 +288,7 @@ impl LayoutFromYamlIntermediate {
             Some("default") => Self::default_from_assets(),
             Some("strider") => Self::strider_from_assets(),
             Some("disable-status-bar") => Self::disable_status_from_assets(),
+            Some("compact") => Self::compact_from_assets(),
             None | Some(_) => Err(ConfigError::IoPath(
                 std::io::Error::new(std::io::ErrorKind::Other, "The layout was not found"),
                 path.into(),
@@ -294,6 +315,12 @@ impl LayoutFromYamlIntermediate {
             serde_yaml::from_str(&String::from_utf8(setup::NO_STATUS_LAYOUT.to_vec())?)?;
         Ok(layout)
     }
+
+    pub fn compact_from_assets() -> LayoutFromYamlIntermediateResult {
+        let layout: LayoutFromYamlIntermediate =
+            serde_yaml::from_str(&String::from_utf8(setup::COMPACT_BAR_LAYOUT.to_vec())?)?;
+        Ok(layout)
+    }
 }
 
 type LayoutFromYamlResult = Result<LayoutFromYaml, ConfigError>;
@@ -421,6 +448,7 @@ pub struct LayoutTemplate {
     #[serde(default)]
     pub body: bool,
     pub split_size: Option<SplitSize>,
+    pub focus: Option<bool>,
     pub run: Option<RunFromYaml>,
 }
 
@@ -466,6 +494,7 @@ pub struct TabLayout {
     pub split_size: Option<SplitSize>,
     #[serde(default)]
     pub name: String,
+    pub focus: Option<bool>,
     pub run: Option<RunFromYaml>,
 }
 
@@ -712,6 +741,7 @@ impl TryFrom<TabLayout> for Layout {
             borderless: tab.borderless,
             parts: Self::from_vec_tab_layout(tab.parts)?,
             split_size: tab.split_size,
+            focus: tab.focus,
             run: tab.run.map(Run::try_from).transpose()?,
         })
     }
@@ -726,6 +756,7 @@ impl From<TabLayout> for LayoutTemplate {
             parts: Self::from_vec_tab_layout(tab.parts),
             body: false,
             split_size: tab.split_size,
+            focus: tab.focus,
             run: tab.run,
         }
     }
@@ -741,6 +772,7 @@ impl TryFrom<LayoutTemplate> for Layout {
             borderless: template.borderless,
             parts: Self::from_vec_template_layout(template.parts)?,
             split_size: template.split_size,
+            focus: template.focus,
             run: template
                 .run
                 .map(Run::try_from)
@@ -761,6 +793,7 @@ impl Default for TabLayout {
             run: None,
             name: String::new(),
             pane_name: None,
+            focus: None,
         }
     }
 }
@@ -778,10 +811,12 @@ impl Default for LayoutTemplate {
                 body: true,
                 borderless: false,
                 split_size: None,
+                focus: None,
                 run: None,
                 parts: vec![],
             }],
             split_size: None,
+            focus: None,
             run: None,
         }
     }
