@@ -1,12 +1,12 @@
 //! Handles cli and configuration options
 use crate::cli::Command;
+use clap::{ArgEnum, Args};
 use serde::{Deserialize, Serialize};
 use std::path::PathBuf;
 use std::str::FromStr;
-use structopt::StructOpt;
 use zellij_tile::data::InputMode;
 
-#[derive(Copy, Clone, Debug, PartialEq, Deserialize, Serialize)]
+#[derive(Copy, Clone, Debug, PartialEq, Deserialize, Serialize, ArgEnum)]
 pub enum OnForceClose {
     #[serde(alias = "quit")]
     Quit,
@@ -32,7 +32,7 @@ impl FromStr for OnForceClose {
     }
 }
 
-#[derive(Clone, Default, Debug, PartialEq, Deserialize, Serialize, StructOpt)]
+#[derive(Clone, Default, Debug, PartialEq, Deserialize, Serialize, Args)]
 /// Options that can be set either through the config file,
 /// or cli flags - cli flags should take precedence over the config file
 /// TODO: In order to correctly parse boolean flags, this is currently split
@@ -40,40 +40,73 @@ impl FromStr for OnForceClose {
 pub struct Options {
     /// Allow plugins to use a more simplified layout
     /// that is compatible with more fonts (true or false)
-    #[structopt(long)]
+    #[clap(long)]
     #[serde(default)]
     pub simplified_ui: Option<bool>,
     /// Set the default theme
-    #[structopt(long)]
+    #[clap(long)]
     pub theme: Option<String>,
     /// Set the default mode
-    #[structopt(long)]
+    #[clap(long, arg_enum, hide_possible_values = true)]
     pub default_mode: Option<InputMode>,
     /// Set the default shell
-    #[structopt(long, parse(from_os_str))]
+    #[clap(long, parse(from_os_str))]
     pub default_shell: Option<PathBuf>,
     /// Set the layout_dir, defaults to
     /// subdirectory of config dir
-    #[structopt(long, parse(from_os_str))]
+    #[clap(long, parse(from_os_str))]
     pub layout_dir: Option<PathBuf>,
-    #[structopt(long)]
+    #[clap(long)]
     #[serde(default)]
     /// Set the handling of mouse events (true or false)
     /// Can be temporarily bypassed by the [SHIFT] key
     pub mouse_mode: Option<bool>,
-    #[structopt(long)]
+    #[clap(long)]
     #[serde(default)]
     /// Set display of the pane frames (true or false)
     pub pane_frames: Option<bool>,
-    #[structopt(long)]
+    #[clap(long)]
     #[serde(default)]
     /// Mirror session when multiple users are connected (true or false)
     pub mirror_session: Option<bool>,
     /// Set behaviour on force close (quit or detach)
-    #[structopt(long)]
+    #[clap(long, arg_enum, hide_possible_values = true)]
     pub on_force_close: Option<OnForceClose>,
-    #[structopt(long)]
+    #[clap(long)]
     pub scroll_buffer_size: Option<usize>,
+
+    /// Switch to using a user supplied command for clipboard instead of OSC52
+    #[clap(long)]
+    #[serde(default)]
+    pub copy_command: Option<String>,
+
+    /// OSC52 destination clipboard
+    #[clap(long, arg_enum, ignore_case = true, conflicts_with = "copy-command")]
+    #[serde(default)]
+    pub copy_clipboard: Option<Clipboard>,
+
+    /// Automatically copy when selecting text (true or false)
+    #[clap(long)]
+    #[serde(default)]
+    pub copy_on_select: Option<bool>,
+
+    /// Explicit full path to open the scrollback editor (default is $EDITOR or $VISUAL)
+    #[clap(long, parse(from_os_str))]
+    pub scrollback_editor: Option<PathBuf>,
+}
+
+#[derive(ArgEnum, Deserialize, Serialize, Debug, Clone, Copy, PartialEq)]
+pub enum Clipboard {
+    #[serde(alias = "system")]
+    System,
+    #[serde(alias = "primary")]
+    Primary,
+}
+
+impl Default for Clipboard {
+    fn default() -> Self {
+        Self::System
+    }
 }
 
 impl Options {
@@ -99,6 +132,12 @@ impl Options {
         let theme = other.theme.or_else(|| self.theme.clone());
         let on_force_close = other.on_force_close.or(self.on_force_close);
         let scroll_buffer_size = other.scroll_buffer_size.or(self.scroll_buffer_size);
+        let copy_command = other.copy_command.or_else(|| self.copy_command.clone());
+        let copy_clipboard = other.copy_clipboard.or(self.copy_clipboard);
+        let copy_on_select = other.copy_on_select.or(self.copy_on_select);
+        let scrollback_editor = other
+            .scrollback_editor
+            .or_else(|| self.scrollback_editor.clone());
 
         Options {
             simplified_ui,
@@ -111,6 +150,10 @@ impl Options {
             mirror_session,
             on_force_close,
             scroll_buffer_size,
+            copy_command,
+            copy_clipboard,
+            copy_on_select,
+            scrollback_editor,
         }
     }
 
@@ -140,6 +183,12 @@ impl Options {
         let theme = other.theme.or_else(|| self.theme.clone());
         let on_force_close = other.on_force_close.or(self.on_force_close);
         let scroll_buffer_size = other.scroll_buffer_size.or(self.scroll_buffer_size);
+        let copy_command = other.copy_command.or_else(|| self.copy_command.clone());
+        let copy_clipboard = other.copy_clipboard.or(self.copy_clipboard);
+        let copy_on_select = other.copy_on_select.or(self.copy_on_select);
+        let scrollback_editor = other
+            .scrollback_editor
+            .or_else(|| self.scrollback_editor.clone());
 
         Options {
             simplified_ui,
@@ -152,6 +201,10 @@ impl Options {
             mirror_session,
             on_force_close,
             scroll_buffer_size,
+            copy_command,
+            copy_clipboard,
+            copy_on_select,
+            scrollback_editor,
         }
     }
 
@@ -164,17 +217,17 @@ impl Options {
     }
 }
 
-#[derive(Clone, Default, Debug, PartialEq, StructOpt, Serialize, Deserialize)]
+#[derive(Clone, Default, Debug, PartialEq, Args, Serialize, Deserialize)]
 /// Options that can be set through cli flags
 /// boolean flags end up toggling boolean options in `Options`
 pub struct CliOptions {
     /// Disable handling of mouse events
-    #[structopt(long, conflicts_with("mouse-mode"))]
+    #[clap(long, conflicts_with("mouse-mode"))]
     pub disable_mouse_mode: bool,
     /// Disable display of pane frames
-    #[structopt(long, conflicts_with("pane-frames"))]
+    #[clap(long, conflicts_with("pane-frames"))]
     pub no_pane_frames: bool,
-    #[structopt(flatten)]
+    #[clap(flatten)]
     options: Options,
 }
 
@@ -200,6 +253,10 @@ impl From<CliOptions> for Options {
             mirror_session: opts.mirror_session,
             on_force_close: opts.on_force_close,
             scroll_buffer_size: opts.scroll_buffer_size,
+            copy_command: opts.copy_command,
+            copy_clipboard: opts.copy_clipboard,
+            copy_on_select: opts.copy_on_select,
+            scrollback_editor: opts.scrollback_editor,
         }
     }
 }
